From e4401e650d05cb86aadcc4d22716b5ce6b6e1e84 Mon Sep 17 00:00:00 2001
From: Dakshinya T R S <dakshinya@multicorewareinc.com>
Date: Fri, 16 Apr 2021 10:40:00 +0530
Subject: [PATCH] Fix VMAF exceptions

---
 doc/reST/api.rst                |  2 ++
 source/CMakeLists.txt           |  2 +-
 source/encoder/api.cpp          |  8 +++++---
 source/encoder/encoder.cpp      | 20 ++++++++++++++++++++
 source/encoder/encoder.h        |  4 ++++
 source/encoder/frameencoder.cpp |  1 +
 source/x265.h                   |  5 +++--
 7 files changed, 36 insertions(+), 6 deletions(-)

diff --git a/doc/reST/api.rst b/doc/reST/api.rst
index 7444f35af..96a62c40f 100644
--- a/doc/reST/api.rst
+++ b/doc/reST/api.rst
@@ -424,6 +424,8 @@ VMAF (Video Multi-Method Assessment Fusion)
 If you set the ENABLE_LIBVMAF cmake option to ON, then x265 will report per frame
 and aggregate VMAF score for the given input and dump the scores in csv file.
 The user also need to specify the :option:`--recon` in command line to get the VMAF scores.
+
+VMAF version support >= 2.0.0
  
     /* x265_calculate_vmafScore:
      *    returns VMAF score for the input video.
diff --git a/source/CMakeLists.txt b/source/CMakeLists.txt
index b4e57b592..f4a9cb793 100755
--- a/source/CMakeLists.txt
+++ b/source/CMakeLists.txt
@@ -29,7 +29,7 @@ option(NATIVE_BUILD "Target the build CPU" OFF)
 option(STATIC_LINK_CRT "Statically link C runtime for release builds" OFF)
 mark_as_advanced(FPROFILE_USE FPROFILE_GENERATE NATIVE_BUILD)
 # X265_BUILD must be incremented each time the public API is changed
-set(X265_BUILD 200)
+set(X265_BUILD 201)
 configure_file("${PROJECT_SOURCE_DIR}/x265.def.in"
                "${PROJECT_BINARY_DIR}/x265.def")
 configure_file("${PROJECT_SOURCE_DIR}/x265_config.h.in"
diff --git a/source/encoder/api.cpp b/source/encoder/api.cpp
index 66f26e37e..5096e52ab 100644
--- a/source/encoder/api.cpp
+++ b/source/encoder/api.cpp
@@ -633,7 +633,9 @@ void x265_vmaf_encoder_log(x265_encoder* enc, int argc, char **argv, x265_param
     if (enc)
     {
         Encoder *encoder = static_cast<Encoder*>(enc);
-        x265_stats stats;       
+        x265_stats stats;
+
+        vmafdata->format = encoder->m_vmafInputFormat;
         stats.aggregateVmafScore = x265_calculate_vmafscore(param, vmafdata);
         if(vmafdata->reference_file)
             fclose(vmafdata->reference_file);
@@ -1925,7 +1927,7 @@ double x265_calculate_vmafscore(x265_param *param, x265_vmaf_data *data)
     else
         x265_log(NULL, X265_LOG_ERROR, "Invalid format\n");
 
-    compute_vmaf(&score, vcd->format, data->width, data->height, read_frame, data, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample, vcd->enable_conf_interval);
+    compute_vmaf(&score, data->format, data->width, data->height, read_frame, data, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample, vcd->enable_conf_interval);
 
     return score;
 }
@@ -2027,7 +2029,7 @@ double x265_calculate_vmaf_framelevelscore(x265_vmaf_framedata *vmafframedata)
         read_frame = read_frame_8bit;
     else
         read_frame = read_frame_10bit;
-    compute_vmaf(&score, vcd->format, vmafframedata->width, vmafframedata->height, read_frame, vmafframedata, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample, vcd->enable_conf_interval);
+    compute_vmaf(&score, vmafframedata->format, vmafframedata->width, vmafframedata->height, read_frame, vmafframedata, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample, vcd->enable_conf_interval);
 
     return score;
 }
diff --git a/source/encoder/encoder.cpp b/source/encoder/encoder.cpp
index c1e1cb46d..3d4742299 100644
--- a/source/encoder/encoder.cpp
+++ b/source/encoder/encoder.cpp
@@ -586,6 +586,22 @@ void Encoder::create()
     fail:
         m_aborted = true;
     }
+#ifdef ENABLE_LIBVMAF
+    m_vmafInputFormat = X265_MALLOC(char, 11);
+    if (m_param->internalCsp == X265_CSP_I420)
+        memcpy(m_vmafInputFormat, "yuv420p", 7);
+    else if (m_param->internalCsp == X265_CSP_I422)
+        memcpy(m_vmafInputFormat, "yuv422p", 7);
+    else if (m_param->internalCsp == X265_CSP_I444)
+        memcpy(m_vmafInputFormat, "yuv444p", 7);
+    else
+        x265_log(NULL, X265_LOG_ERROR, "Invalid format\n");
+
+    if (m_param->internalBitDepth == 10)
+        strcat(m_vmafInputFormat, "10le");
+    else if (m_param->internalBitDepth == 12)
+        strcat(m_vmafInputFormat, "12le");
+#endif
 }
 
 void Encoder::stopJobs()
@@ -999,6 +1015,10 @@ void Encoder::destroy()
 #ifdef SVT_HEVC
     X265_FREE(m_svtAppData);
 #endif
+
+#ifdef ENABLE_LIBVMAF
+    X265_FREE(m_vmafInputFormat);
+#endif
     if (m_param)
     {
         if (m_param->csvfpt)
diff --git a/source/encoder/encoder.h b/source/encoder/encoder.h
index 2ee5bdaee..d0b74db70 100644
--- a/source/encoder/encoder.h
+++ b/source/encoder/encoder.h
@@ -296,6 +296,10 @@ public:
     ThreadSafeInteger* zoneReadCount;
     ThreadSafeInteger* zoneWriteCount;
 
+#ifdef ENABLE_LIBVMAF
+    char* m_vmafInputFormat;
+#endif
+
     Encoder();
     ~Encoder()
     {
diff --git a/source/encoder/frameencoder.cpp b/source/encoder/frameencoder.cpp
index 1b3875a25..11221078d 100644
--- a/source/encoder/frameencoder.cpp
+++ b/source/encoder/frameencoder.cpp
@@ -2151,6 +2151,7 @@ void FrameEncoder::vmafFrameLevelScore()
     vmafframedata->internalBitDepth = m_param->internalBitDepth;
     vmafframedata->reference_frame = fenc;
     vmafframedata->distorted_frame = recon;
+    vmafframedata->format = m_top->m_vmafInputFormat;
 
     fenc->m_vmafScore = x265_calculate_vmaf_framelevelscore(vmafframedata);
 
diff --git a/source/x265.h b/source/x265.h
index 324f3163f..a37741f53 100644
--- a/source/x265.h
+++ b/source/x265.h
@@ -712,6 +712,7 @@ typedef struct x265_vmaf_data
     int internalBitDepth;
     FILE *reference_file; /* FILE pointer for input file */
     FILE *distorted_file; /* FILE pointer for recon file generated*/
+    char *format;
 }x265_vmaf_data;
 
 /* data to calculate frame level VMAF score */
@@ -723,12 +724,12 @@ typedef struct x265_vmaf_framedata
     int internalBitDepth; 
     void *reference_frame; /* points to fenc of particular frame */
     void *distorted_frame; /* points to recon of particular frame */
+    char *format;
 }x265_vmaf_framedata;
 
 /* common data needed to calculate both frame level and video level VMAF scores */
 typedef struct x265_vmaf_commondata
 {
-    char *format;
     char *model_path;
     char *log_path;
     char *log_fmt;
@@ -745,7 +746,7 @@ typedef struct x265_vmaf_commondata
     int enable_conf_interval;
 }x265_vmaf_commondata;
 
-static const x265_vmaf_commondata vcd[] = { { NULL, (char *)"/usr/local/share/model/vmaf_v0.6.1.pkl", NULL, NULL, 0, 0, 0, 0, 0, 0, 0, NULL, 0, 1, 0 } };
+static const x265_vmaf_commondata vcd[] = { { (char *)"/usr/local/share/model/vmaf_v0.6.1.json", NULL, NULL, 0, 0, 0, 0, 0, 0, 0, NULL, 0, 1, 0 } };
 
 /* x265 input parameters
  *
-- 
2.17.1

